/*#############################################################
File Name       : ex3_HC06_leds_sw_analog.c
Author          : Grant Phillips
Date Modified   : 21/04/2014
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)
Tested On       : STM32F3-Discovery

Description     : Example program that uses the UART5 module to 
									establish a serial link via a HC-06 Bluetooth
									module with another device.  The purpose of the 
									STM32F3 is to control 8 LEDs, read a pushbutton, 
									and read an analog input.  
									The message protocol is as follows:
              
                   byte (HEX) byte (HEX) byte(HEX)  byte(HEX) byte(HEX)
                  ------------------------------------------------------
                  | COMMAND  |   DATA2  |   DATA1  |  DATA0  |    LF   |
                  ------------------------------------------------------
                  
									(from other device to this STM32F3)
                  COMMAND  DESCRIPTION
                  =======  ===========
                  00       Set the LEDs to the value of DATA2.  
                           DATA1 and DATA0 is not used
                  01       Set the LED flash rate to the value
                           of DATA2.  DATA1 and DATA0 is not used.
                  02       Flash the LEDs.  DATA2, DATA1 and DATA0 are 
                           not used.
                  03       Stop flashing the LEDs.  DATA2, DATA1 and 
                           DATA0 are not used.
									04       Request the value of the pushbutton. 
                           DATA2, DATA1 and DATA0 are not used.
									06       Request the value of the analog input. 
                           DATA2, DATA1 and DATA0 are not used.
									08       Request the value of BOTH the pushbutton and
													 the analog input.  DATA2, DATA1 and DATA0 are 
													 not used.
													 
									(from this STM32F3 to other device)
                  COMMAND  DESCRIPTION
                  =======  ===========
                  05       Reply from the STM32F3 with pushbutton
													 value. DATA2 contains the value. DATA1 
                           and DATA0 are not used.
                  07       Reply from the STM32F3 with analog value. 
                           DATA2 contains the high byte of the 
                           12-bit value and DATA1 the low byte. 
													 DATA0 is not used.
									09       Reply from the STM32F3 with BOTH the 
													 pushbutton value and the analog value. 
                           DATA2 contains the pushbutton value. DATA1 
													 contains the high byte of the 12-bit value 
													 and DATA0 the low byte. 						

                  E.g.
                  If the pushbutton is on, and the other device sends
                  the command: "04", then the reply from the STM32F3 
									will be:  "0501"; when it was off it would be "0500"
                  
                  E.g.
                  If the analog value is at maximum (4095 or
                  0x0FFF), and the other device sends the command: 
                  "06", then the reply from the STM32F3 will 
                  be:  "0703FF"
									
									E.g.
									If the pushbutton is on and the analog value is at 
									maximum, and the other device sends the command:
									"08" to get BOTH the pushbutton value and analog value
									in one reply, then the reply from the STM32F3 will
									be: "09010FFF"
                  
                  Note that all values must be entered in 
                  CAPITAL.

Requirements    : * STM32F3-Discovery Board
									* Make sure the HC06_F3.c file is in the 
                    same folder than the project
              
Circuit         : * 8 LEDs connected to PE15 - PE8 (LD3 - LD10 
                    ON STM32F3-DISCOVERY BOARD)
									* a pushbutton connected to PA0 (USER BUTTON
										STM32F3-DISCOVERY BOARD)
									* a potentiometer (POT) connected to PF4 
                    (ADC1_IN5 - Analog1 Channel 5) 
									* HC06 Bluetooth module is connected as follows:
										TXD			-		PD2
										RXD			-		PC12
										GND			-		GND
										VCC			-		3.3V
									
See the STM32F3-Discovery User Manual (UM1570) for the block 
diagram of the STM32F303VCT6 processor (p13), a summary of
the GPIO connections (p21-29) and the schematic diagram (p31-34)

##############################################################*/

/* #includes -------------------------------------------------*/
#include "main.h"																								//main library to include for device drivers, peripheral drivers, etc.
#include "HC06_F3.c"																						//for the HC06 functions on the STM32F3-Discovery

/* #defines --------------------------------------------------*/

/* #function prototypes --------------------------------------*/
void InitLEDs(void);																					  //prototype for the user function to initialize the USER LEDs
unsigned char HexStToByte(char *St3);														//funtion to convert a HEX string to a byte value
void IntToHexStr(unsigned int data,char *St5);									//function to convert a 16-bit value to a HEX string
void CharToHexStr(unsigned int data,char *St3);									//function to convert a 8-bit value to a HEX string
void InitAtoD(void);																						//prototype for the user function to initialize the ADC
void InitPB(void);																							//prototype for the user function to initialize the Pushbutton

/* #global variables -----------------------------------------*/

RCC_ClocksTypeDef 			RCC_Clocks;															//structure used for setting up the SysTick Interrupt

				
	
// Unused global variables that have to be included to ensure correct compiling */
// ###### DO NOT CHANGE ######
// ===============================================================================
__IO uint32_t TimingDelay = 0;																	//used with the Delay function
__IO uint8_t DataReady = 0;
__IO uint32_t USBConnectTimeOut = 100;
__IO uint32_t UserButtonPressed = 0;
__IO uint8_t PrevXferComplete = 1;
// ===============================================================================


	
int main(void)
{
	unsigned char COMMAND, SETTINGS;
	unsigned char ledvalue=0X00;  		//default LEDs value
	unsigned char leddelay=100;   		//default flash delay
	unsigned char ledflash=0;     		//flag 0 = no flash, 1 = flash
	unsigned char ledtoggle=0;
	unsigned char flashcounter=0;			//counter to keep track of how many ms have passed
	char sendbuf[20];									//buffer string used for sending a message
	char buf[20];											//buffer string used for HEX and INT conversions
	char buf2[20];										//buffer string used for HEX and INT conversions
	unsigned int ADC_val;							//variable for holding the 12-bit ADC result
	unsigned char pb_val;         		//8-bit variable to hold the switches value
	
	/* Set the SysTick Interrupt to occur every 1ms) */
	RCC_GetClocksFreq(&RCC_Clocks);
	if (SysTick_Config(RCC_Clocks.HCLK_Frequency / 1000))
		while(1);																										//will end up in this infinite loop if there was an error with Systick_Config
	
	HC06_Init(115200);																						//initialize the HC-06 module at 115200 baud
	InitLEDs();																									  //initialize the USER LEDs for this application
	InitAtoD();																										//initialize the ADC for this application
	InitPB();																											//initialize the USER button for this application
	
	HC06_ClearRxBuffer();																					//clear the rx buffer after initialization
	
	/* Main program loop */
  while (1)
  {
		/* Test EOC (EndOfConversion flag. Wait for the ADC to finish converting a analog input */
		while(ADC_GetFlagStatus(ADC1, ADC_FLAG_EOC) == RESET);
    
		/* Get ADC1 converted data */
		ADC_val = ADC_GetConversionValue(ADC1);
		
		/* Read the pushbutton value */
    pb_val = GPIO_ReadInputDataBit(GPIOA, GPIO_Pin_0); 
		
		if (ledflash == 1)
		{
			Delay(1);														//delay 1 ms
			flashcounter++;
			if(flashcounter == leddelay)				
			{
				flashcounter=0;
				if(ledtoggle == 0)
				{
					ledtoggle=1;
					GPIO_Write(GPIOE, 0x00<<8);			//clear LEDs
				}
				else
				{
					ledtoggle=0;
					GPIO_Write(GPIOE, ledvalue<<8);	//write ledvalue to the LEDs
				}	
			}
		}
		if(new_HC06_msg == 1)																				//is there a new message?
		{
			/* HC06_msg[0..1] contains the COMMAND */
			buf[0] = HC06_msg[0];
			buf[1] = HC06_msg[1];
			buf[2] = '\0';																						//buf contains COMMAND HEX string
			COMMAND = HexStToByte(buf);																//convert buf to a value
			if (COMMAND == 0)																					//if COMMAND = 0 (SET_LEDS)
			{
				/* HC06_msg[2..3] contains the SETTINGS */
				buf[0] = HC06_msg[2];
				buf[1] = HC06_msg[3];
				buf[2] = '\0';																					//buf contains SETTINGS HEX string
				SETTINGS = HexStToByte(buf);														//convert buf to a value
				ledvalue = SETTINGS;
				GPIO_Write(GPIOE, SETTINGS<<8);													//write the SETTINGS value to the LEDs; LEDs are on MSByte, hence the << 8
			}
			else if (COMMAND == 1)																		//if COMMAND = 1 (SET_FLASH_DELAY)
			{
				buf[0] = HC06_msg[2];
				buf[1] = HC06_msg[3];
				buf[2] = '\0';
				SETTINGS = HexStToByte(buf);
				leddelay = SETTINGS;
			}
			else if (COMMAND == 2)																		//if COMMAND = 2 (START_FLASH)
			{
				ledflash=1;
				flashcounter=0;
			}
			else if (COMMAND == 3)																		//if COMMAND = 3 (STOP_FLASH)
			{
				ledflash=0;
			}
			else if (COMMAND == 4)																		//if COMMAND = 4 (REQUEST_PB)
			{
				CharToHexStr(pb_val, buf);         
				//we could have used printf("05%X%c", sw_val, 10), but according to our
				//protocol all HEX values must have a leading 0 if it is less than 2 characters wide.
				//CharToHexStr() creates a string from a byte value and automatically adds a leading
				//0 where necessary
				sprintf(sendbuf, "05%s\n", buf); 				//write the cmd 05, the pb_val (with leading
																								//0 if necessary), and the NL character to a string buf
				HC06_PutStr(sendbuf);
			}
			else if (COMMAND == 6)																		//if COMMAND = 6 (REQUEST_ANALOG)
			{
				IntToHexStr(ADC_val, buf);          		//same as CharToHexStr, but writes a integer (16-bits)
																								//to string with leading 0's where required
				sprintf(sendbuf, "07%s\n", buf);  		  //write the cmd 07, the ADC_Val (with leading
																								//0 if necessary), and the NL character to a string buf
				HC06_PutStr(sendbuf);
			}
			else if (COMMAND == 8)																		//if COMMAND = 6 (REQUEST_PB_ANALOG)
			{
				CharToHexStr(pb_val, buf);         
				IntToHexStr(ADC_val, buf2);          	
				sprintf(sendbuf, "09%s%s\n", buf, buf2);  		//write the cmd 09, the pb-val, the ADC_Val (with leading
																											//0 if necessary), and the NL character to a string buf
				HC06_PutStr(sendbuf);
			}
			else
			{
				//incorrect command
			}
			new_HC06_msg = 0;																					//clear the flag to indicate that the new message was serviced
		}
  }	
}



void InitLEDs(void)
{
	GPIO_InitTypeDef        GPIO_InitStructure;										//structure used for setting up a GPIO port
	
  /* GPIOE Periph clock enable */
  RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOE, ENABLE);           
  
  /* Configure PE15 - PE8 in output pushpull mode */
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_15 | GPIO_Pin_14 | GPIO_Pin_13 | GPIO_Pin_12 | GPIO_Pin_11 | GPIO_Pin_10 | GPIO_Pin_9 | GPIO_Pin_8;   //which pins to setup, seperated by |
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;               
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;                       
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;                    
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;                     
  GPIO_Init(GPIOE, &GPIO_InitStructure);                                    
}


void InitPB(void)
{
	GPIO_InitTypeDef        GPIO_InitStructure;										//structure used for setting up a GPIO port
	
  /* GPIOA Periph clock enable */
  RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA, ENABLE);
	
	/* Configure PA0 in input mode */
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;									
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;							
  GPIO_Init(GPIOA, &GPIO_InitStructure);
}



void InitAtoD(void)
{
	GPIO_InitTypeDef        GPIO_InitStructure;											//structure used for setting up a GPIO port
	ADC_InitTypeDef   			ADC_InitStructure;											//structure used for setting up the ADC
  ADC_CommonInitTypeDef 	ADC_CommonInitStructure;							  //structure used for setting up the common ADC setting
	
	__IO unsigned int ADC_calibration_value;
	
	/* Configure the ADC clock to be PLLCLK/2 */
  RCC_ADCCLKConfig(RCC_ADC12PLLCLK_Div2);
  
  /* Enable ADC1 clock */
  RCC_AHBPeriphClockCmd(RCC_AHBPeriph_ADC12, ENABLE);
	
						
	/* ADC Channel configuration */
  /* GPIOF Periph clock enable */
  RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOF, ENABLE);

  /* Configure ADC1 Channel5 (PF4) as analog input */
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_4 ;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AN;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL ;
  GPIO_Init(GPIOF, &GPIO_InitStructure);
  
	/* Fill each ADC_InitStructure member with its default value */
  ADC_StructInit(&ADC_InitStructure);

  /* Calibration procedure */  
  ADC_VoltageRegulatorCmd(ADC1, ENABLE);
  
  /* Insert delay equal to 10 s */
  Delay(1);
		
	ADC_SelectCalibrationMode(ADC1, ADC_CalibrationMode_Single);										//select calibratio mode for a single channel
  ADC_StartCalibration(ADC1);																											//start the calibration on ADC1
  
  while(ADC_GetCalibrationStatus(ADC1) != RESET );																//wait for the calibration to end
  ADC_calibration_value = ADC_GetCalibrationValue(ADC1);													//get the calibration value
     
  ADC_CommonInitStructure.ADC_Mode = ADC_Mode_Independent;                        //ADC independent mode                                
  ADC_CommonInitStructure.ADC_Clock = ADC_Clock_AsynClkMode;                    	//ADC Asynchronous clock mode
  ADC_CommonInitStructure.ADC_DMAAccessMode = ADC_DMAAccessMode_Disabled;         //DMA mode disabled    
  ADC_CommonInitStructure.ADC_DMAMode = ADC_DMAMode_OneShot;                  		//ADC DMA Oneshot mode - but not relevant because DMA disabled
  ADC_CommonInitStructure.ADC_TwoSamplingDelay = 0;          											//0 Delay between 2 sampling phases
  ADC_CommonInit(ADC1, &ADC_CommonInitStructure);																	//Initializes ADC1 peripherals according to the specified parameters in the ADC_CommonInitStructure
  
  ADC_InitStructure.ADC_ContinuousConvMode = ADC_ContinuousConvMode_Enable;				//enable continous mode
  ADC_InitStructure.ADC_Resolution = ADC_Resolution_12b; 													//12-bit resolution
  ADC_InitStructure.ADC_ExternalTrigConvEvent = ADC_ExternalTrigConvEvent_0;      //ADC external trigger event 0 
  ADC_InitStructure.ADC_ExternalTrigEventEdge = ADC_ExternalTrigEventEdge_None;		//ADC No external trigger for regular conversion
  ADC_InitStructure.ADC_DataAlign = ADC_DataAlign_Right;													//ADC Data alignment right 
  ADC_InitStructure.ADC_OverrunMode = ADC_OverrunMode_Disable;   									//overrun management disabled
  ADC_InitStructure.ADC_AutoInjMode = ADC_AutoInjec_Disable;  										//disable automatic injected group conversion after regular group conversion
  ADC_InitStructure.ADC_NbrOfRegChannel = 1;																			//Specifies the number of ADC channels that will be converted using the sequencer for regular channel group
  ADC_Init(ADC1, &ADC_InitStructure);																							//Initializes ADC1 peripheral according to the specified parameters in the ADC_InitStruct
  
  /* ADC1 regular channel5 configuration */ 
  ADC_RegularChannelConfig(ADC1, ADC_Channel_5, 1, ADC_SampleTime_7Cycles5);			//Sample time equal to 7.5 cycles
   
  /* Enable ADC1 */
  ADC_Cmd(ADC1, ENABLE);
  
  /* wait for ADRDY */
  while(!ADC_GetFlagStatus(ADC1, ADC_FLAG_RDY));
  
  /* Start ADC1 Software Conversion */ 
  ADC_StartConversion(ADC1);  	
}



//////////////////////////////////////////////////////////////////
//                                                              //
// Function name   : HexStrToByte                               //
//                                                              //
// Original auther : Riaan Ehlers                               //
// Date Modufied   : 04/04/2012                                 //
// Description     : Converts a 3 byte string to a byte         //
//                   eg. "E0" -> 224=0xE0                       //
//                       "12" ->  18=0x12                       //
// Note            : The string must have a string length of 3. //
//                   The string must be 2 uppercase ascii       //
//                   characters followed by an optional null    //
//                   terminator.                                //
//                   Values like "3f" is not seen as hex.       //
//                                                              //
// Compiler        : MPLAB C30 C Copiler ver 3.24               //
//////////////////////////////////////////////////////////////////

unsigned char HexStToByte(char *St3) //eg. char St3[3]="3F" -> return 0x3F
{
 unsigned char Ans=0;

//Higher Nibble
 if(St3[0]<='9') //'0'..'9'
  Ans=(St3[0]-'0')*0x10;
 else
  Ans=(St3[0]-0x37)*0x10; //  Ans+=(St[i]-'A'+0x0A)*0x10;

//Higher Lower
 if(St3[1]<='9') //'0'..'9'
  Ans+=(St3[1]-'0');
 else
  Ans+=(St3[1]-0x37); //  Ans+=(St[i]-'A'+0x0A)*0x01;

 return(Ans);
}



//////////////////////////////////////////////////////////////////
//                                                              //
// Function name   : IntToHexStr                                 //
//                                                              //
// Original auther : Morne Odendaal & Riaan Ehlers              //
// Date Modufied   : 30/06/2004                                 //
// Description     : Converts a 16bit unsigned integer          //
//                   value to a string.                         //
//                   eg. 57515=0xE0AB -> "E0AB"                 //
//                   Min size of St5=5 bytes (char St5[5])      //
//                                                              //
// Note            : rather use printf("%x") for CCS compiler   //
// Compiler        : MPLAB C18 C Copiler ver 2.20               //
//////////////////////////////////////////////////////////////////

void IntToHexStr(unsigned int data,char *St5)
{
 char nibble;
 unsigned int place,x;
 char StPtr=0;

 x=16;
 for (place = 0xF000;place >= 0x000F;place=place>>4)
 {           
  x=x - 0x4;
  nibble = ((data & place) >> x) + 0x30; //0x30='0'
  if (nibble > 0x39)                     //if(nibble>'9')
  {
   nibble = nibble + 0x07;               //Add ascii offset. There are 7 characters between '9' and 'A'
  }//if ((nibble != 0) && (place == 0xF000)) 

  St5[StPtr]=nibble;
  StPtr++;
 }//for
 St5[4]='\0'; //Terminate string
}



//////////////////////////////////////////////////////////////////
//                                                              //
// Function name   : CharToHexStr                               //
//                                                              //
// Tested by       : Riaan Ehlers                               //
// Date Modified   : 18/06/2012                                 //
// Comment         : Tested with MPLAB C18 C Compiler ver 3.36  //
//                                                              //
// Original Auther : Riaan Ehlers                               //
// Date Modified   : 28/02/2012                                 //
// Description     : Convert the lower byte of a 16bit unsigned //
//                   integer to a string.                       //
//                   eg. 249=0xF9 -> "F9" (Good for Bin to text)//
//                       153=0x99 -> "99" (Good for BCD to text)//
//                   Min size of St3=3 bytes (char St3[3])      //
// Compiler        : MPLAB C30 C Compiler ver 3.24              //
//                                                              //
//////////////////////////////////////////////////////////////////

void CharToHexStr(unsigned int data,char *St3) //char St3[3];
{
 int nibble;

 nibble = ((data & 0xF0) >> 4) + '0';
 if (nibble > '9')
 {
  nibble = nibble + 0x07;               //Add ascii offset. There are 7 characters between '9' and 'A'
 } 
 St3[0]=nibble; //'0'..'F'

 nibble = (data & 0x0F) + '0';
 if (nibble > '9')
 {
  nibble = nibble + 0x07;               //Add ascii offset. There are 7 characters between '9' and 'A'
 } 
 St3[1]=nibble; //'0'..'F'

 St3[2]='\0'; //Terminate string
}



// -------------------------------------------------------------------------------

// Function to insert a timing delay of nTime
// ###### DO NOT CHANGE ######
void Delay(__IO uint32_t nTime)
{
  TimingDelay = nTime;

  while(TimingDelay != 0);
}

// Function to Decrement the TimingDelay variable.
// ###### DO NOT CHANGE ######
void TimingDelay_Decrement(void)
{
  if (TimingDelay != 0x00)
  { 
    TimingDelay--;
  }
}

// Unused functions that have to be included to ensure correct compiling
// ###### DO NOT CHANGE ######
// =======================================================================
uint32_t L3GD20_TIMEOUT_UserCallback(void)
{
  return 0;
}

uint32_t LSM303DLHC_TIMEOUT_UserCallback(void)
{
  return 0;
}
// =======================================================================



